<?php

namespace Drupal\erl_restriction\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\entity_reference_layout\Plugin\Field\FieldWidget\EntityReferenceLayoutWidget;

/**
 * Entity Reference with Layout field widget.
 *
 * @FieldWidget(
 *   id = "erl_restriction_widget",
 *   label = @Translation("Entity reference layout (With layout builder and restriction)"),
 *   description = @Translation("Layout builder for paragraphs."),
 *   field_types = {
 *     "entity_reference_layout_revisioned"
 *   },
 * )
 */
class ErlRestrictionWidget extends EntityReferenceLayoutWidget implements ContainerFactoryPluginInterface {

  /**
   * Builds the main widget form array container/wrapper.
   *
   * Form elements for individual items are built by formElement().
   */
  protected function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {

    $elements = parent::formMultipleElements($items, $form, $form_state);

    foreach ($elements as $key => $layout) {

      if (!is_numeric($key) || !isset($layout['layout']['#value']) || !isset($elements['add_more']['type']['#options'])) {
        continue;
      }

      // Get the name of the layout (eg. layout_image_right).
      $layout = $layout['layout']['#value'];

      if (!isset($this->getSettings()['restrictions'][$layout])) {
        continue;
      }
      // Allowed paragraphs in field.
      $option_keys = [];
      foreach ($elements['add_more']['type']['#options'] as $machine_name => $name) {
        $option_keys[] = $machine_name;
      }

      $regions = $this->getSettings()['restrictions'][$layout];

      foreach ($regions as $region_key => $region) {

        $selected = [];
        foreach ($regions[$region_key] as $value_key => $value) {
          if ($value !== 0) {
            $selected[] = $value_key;
          }
        }
        $intersect = array_intersect($selected, $option_keys);

        $allowed = [];

        // Allowed in both, field and specific layout.
        foreach ($intersect as $machine_name) {
          $allowed[$machine_name] = $elements['add_more']['type']['#options'][$machine_name];
        }

        $elements[$key][$region_key]['allowed'] = [
          '#type' => 'select',
          '#options' => $allowed,
          '#attributes' => [
            'class' => ['erl-item-type', 'hidden'],
            'id' => $elements["#field_name"] . '--item-' .$key . '-' . $region_key . '-select',
          ],
        ];
      }
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $defaults = parent::defaultSettings();
    $defaults += [
      'restrictions' => [],
    ];

    return $defaults;
  }

  /**
   * {@inheritdoc}
   */
  function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $field_settings = $this->fieldDefinition->getSetting('handler_settings');
    $paragraphs = $this->entityTypeBundleInfo->getBundleInfo('paragraph');

    $options = [];

    foreach ($paragraphs as $key => $paragraph) {
      $options[$key] = $paragraph['label'];
    }

    $form['restrictions'] = [];

    $definitions = $this->layoutPluginManager->getDefinitions();

    foreach ($field_settings['allowed_layouts'] as $allowed_layout_type) {
      foreach ($allowed_layout_type as $layout_key => $layout_name) {

        $form['restrictions'][$layout_key] = [
          '#type' => 'fieldset',
          '#title' => $layout_name,
        ];

        foreach ($definitions[$layout_key]->getRegions() as $region_key => $region) {
          $default = '';
          if (isset($this->getSettings()['restrictions'][$layout_key][$region_key])) {
            $default = $this->getSettings()['restrictions'][$layout_key][$region_key];
          }

          $form['restrictions'][$layout_key][$region_key] = [
            '#type' => 'checkboxes',
            '#title' => $region['label'],
            '#options' => $options,
            '#default_value' => $default,
          ];
        }
      }
    }
    return $form;
  }
}
